package com.bearprogrammer.blog.cs.util;

/**
 * Logic related to prime numbers.
 * 
 * @author Vinicius Isola
 */
public class PrimeNumber {
	
	/**
	 * Return the next prime after <code>afterNumber</code>
	 * 
	 * @param afterNumber
	 *            The search will start after this number.
	 * @return The next prime found.
	 */
	public static int getNextPrime(int afterNumber) {
		int number = afterNumber + 1;
		while (!isPrime(number)) {
			number++;
		}
		return number;
	}

	/**
	 * Check if a number is prime.
	 * 
	 * @param number
	 *            The number to check for.
	 * @return True if the number is prime, false otherwise.
	 * @see http://stackoverflow.com/questions/9625663/calculating-and-printing-the-nth-prime-number
	 */
	public static boolean isPrime(int number) {
		for (int i = 2; i <= Math.sqrt(number); i++) {
			if (number % i == 0) {
				return false;
			}
		}
		return true;
	}
	
	/**
	 * Check if a number is prime, the slow/naive way.
	 * 
	 * @param number
	 *            The number to check for.
	 * @return True if the number is prime, false otherwise.
	 */
	public static boolean slowIsPrime(int number) {
		for (int i = number - 1; i > 1; i--) {
			if (number % i == 0) {
				return false;
			}
		}
		return true;
	}
	
	/**
	 * The last prime number found in this instance.
	 */
	private int lastFound = 1;
	
	/**
	 * Do nothing constructor.
	 */
	public PrimeNumber() {
		
	}
	
	/**
	 * Start this instance with a higher prime number to make it more efficient.
	 * 
	 * @param startFrom
	 *            The number to start from. It will be checked to be a prime
	 *            number.
	 */
	public PrimeNumber(int startFrom) {
		if (!isPrime(startFrom)) {
			throw new IllegalArgumentException(startFrom + " is not a prime number.");
		}
		lastFound = startFrom;
	}
	
	/**
	 * Return the last prime number found.
	 * 
	 * @return Last prime found in this instance.
	 */
	public int getLastFound() {
		return lastFound;
	}
	
	/**
	 * Return the next prime after <code>afterNumber</code>
	 * 
	 * @return The next prime found.
	 */
	public int getNextPrime() {
		return lastFound = getNextPrime(lastFound);
	}

}
