package com.bearprogrammer.blog.cs.datastructure;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

public class HashTableTest {
	
	private static final int ELEMENTS = 1000;
	private static final String ANOTHER_ELEMENT = "Some other text";
	private static final String ELEMENT = "Some text";
	
	private HashTable<String> hashTable;
	private List<String> bigCollection = new ArrayList<String>();
	
	@Before
	public void setup() {
		hashTable = new HashTable<String>();
		
		// Add count elements
		for (Integer i = 0; i < ELEMENTS; i++) {
			bigCollection.add(i.toString());
		}
	}
	
	@Test
	public void addAllShouldAddElementsOnlyOnceAndReturnTrueIfChanged() {
		boolean changed = hashTable.addAll(bigCollection);
		Assert.assertTrue("Should return true if changed.", changed);
		Assert.assertEquals("Should contain " + ELEMENTS + " elements.", ELEMENTS, hashTable.size());
	}
	
	@Test
	public void addAllShouldNotAddExistingElementsAndReturnFalseIfNotChanged() {
		hashTable.addAll(bigCollection);
		
		// Add all again
		boolean changed = hashTable.addAll(bigCollection);
		Assert.assertFalse("Should return false if not changed.", changed);
		Assert.assertEquals("Should still contain " + ELEMENTS + " elements.", ELEMENTS, hashTable.size());
	}
	
	@Test
	public void addShouldReturnFalseWhenAddingExistingElements() {
		for (Integer i = 0; i < ELEMENTS; i++) {
			hashTable.add(i.toString());
		}
		
		for (Integer i = 0; i < ELEMENTS; i++) {
			boolean added = hashTable.add(i.toString());
			Assert.assertFalse("Should not return true if element was not added.", added);
		}
		Assert.assertEquals("Should still contain " + ELEMENTS + " elements.", ELEMENTS, hashTable.size());
	}
	
	@Test
	public void addShouldReturnTrueWhenAddingNewElements() {
		for (Integer i = 0; i < ELEMENTS; i++) {
			boolean added = hashTable.add(i.toString());
			Assert.assertTrue("Should return true if element was added.", added);
		}
		Assert.assertEquals("Should contain " + ELEMENTS + " elements.", ELEMENTS, hashTable.size());
	}
	
	@Test(expected=NullPointerException.class)
	public void addShouldThrowExceptionWhenAddingNull() {
		hashTable.add(null);
	}
	
	@Test
	public void containsShouldReturnFalseForObjectNotAdded() {
		hashTable.add(ELEMENT);
		Assert.assertFalse("Should not contain object that wasn't added.", hashTable.contains(ANOTHER_ELEMENT));
	}
	
	@Test
	public void containsShouldReturnTrueForAddedElement() {
		hashTable.add(ELEMENT);
		Assert.assertTrue("Should contain specified object.", hashTable.contains(ELEMENT));
	}
	
	@Test
	public void iteratorShouldRemoveElement() {
		hashTable.addAll(bigCollection);
		
		int counter = 0;
		for (Iterator<String> iterator = hashTable.iterator(); iterator.hasNext();) {
			iterator.next();
			if (counter % 2 == 0) {
				iterator.remove();
			}
			counter++;
		}
		
		Assert.assertEquals("Should have returned all elements in HashTable.", ELEMENTS, counter);
		Assert.assertEquals("Should have removed half elements.", ELEMENTS / 2, hashTable.size());
	}
	
	@Test
	@SuppressWarnings("unused")
	public void iteratorShouldReturnAllElements() {
		hashTable.addAll(bigCollection);
		
		for (String element : hashTable) {
			bigCollection.remove(element);
		}
		
		Assert.assertEquals("Iterator should return all elements in the HashTable.", 0, bigCollection.size());
		
		hashTable = new HashTable<String>();
		
		hashTable.add("One");
		hashTable.add("Two");
		hashTable.add("Three");
		hashTable.add("Four");
		hashTable.add("Five");
		
		int count = 0;
		for (String element : hashTable) {
			count++;
		}
		
		Assert.assertEquals("Iterator should return all elements in the HashTable.", 5, count);
	}
	
	@Test
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public void performanceComparisonWithOtherSets() {
		Class<? extends Set> [] setClasses = new Class[] {HashTable.class, HashSet.class, TreeSet.class, LinkedHashSet.class};
		try {
			for (int i = 10; i <= 1000; i *= 10) {
				System.out.println("--- Count " + i + " ---");
				for (Class<? extends Set> setClass : setClasses) {
					String name = setClass.getSimpleName();
					while (name.length() < 20) name = " " + name;
					
					String addMany = addMany(setClass, i) + "ns";
					while (addMany.length() < 12) addMany = " " + addMany;
					
					String removeMany = removeMany(setClass, i) + "ns";
					while (removeMany.length() < 12) removeMany = " " + removeMany;
					
					System.out.println(name + ", Add many: " + addMany + ", Remove Many: " + removeMany);
				}
			}
			System.out.println("------------------------");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private long addMany(Class<? extends Set> setClass, long count) throws InstantiationException, IllegalAccessException {
		long totalTime = 0;
		for (int i = 0; i < count; i++) {
			Set<String> set = setClass.newInstance();
			for (String e : bigCollection) {
				long start = System.nanoTime();
				set.add(e);
				totalTime += System.nanoTime() - start;
			}
		}
		return totalTime / (count * bigCollection.size());
	}
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private long removeMany(Class<? extends Set> setClass, long count) throws InstantiationException, IllegalAccessException {
		long totalTime = 0;
		for (int i = 0; i < count; i++) {
			Set<String> set = setClass.newInstance();
			set.addAll(bigCollection);
			for (String e : bigCollection) {
				long start = System.nanoTime();
				set.remove(e);
				totalTime += System.nanoTime() - start;
			}
		}
		return totalTime / (count * bigCollection.size());
	}
	
	@Test
	public void removeExistingElementShouldReturnTrue () {
		hashTable.add(ELEMENT);
		Assert.assertTrue("Should return true if removing existing element.", hashTable.remove(ELEMENT));
		
		hashTable.addAll(bigCollection);
		Assert.assertTrue("Should return true if removing existing element.", hashTable.remove("350"));
		Assert.assertTrue("Should return true if removing existing element.", hashTable.remove("2"));
		Assert.assertTrue("Should return true if removing existing element.", hashTable.remove("777"));
	}
	
	@Test
	public void removeNonExistingElementShouldReturnFalse () {
		hashTable.add(ELEMENT);
		Assert.assertFalse("Should return false if removing non-existing element.", hashTable.remove(ANOTHER_ELEMENT));
		
		hashTable.addAll(bigCollection);
		Assert.assertFalse("Should return false if removing non-existing element.", hashTable.remove("WILL_NOT_FIND"));
		Assert.assertFalse("Should return false if removing non-existing element.", hashTable.remove("ALSO_NOT_THERE"));
		Assert.assertFalse("Should return false if removing non-existing element.", hashTable.remove("HA_FORGET_IT"));
	}
	
	@Test
	public void removeNullReturnsFalse() {
		Assert.assertFalse("Should return false if removing null.", hashTable.remove(null));
	}
	
	@Test
	public void toArrayShouldReturnAllElements() {
		String [] elements = {"One", "Two", "Three", "Four", "Five", "Six", "Seven", "Eight", "Nine", "Ten"};
		
		for (String e : elements) {
			hashTable.add(e);
		}
		
		String [] returned = hashTable.toArray(new String[hashTable.size()]);
		
		Arrays.sort(elements);
		Arrays.sort(returned);
		Assert.assertEquals("To array should return all elements added.", elements.length, returned.length);
		Assert.assertArrayEquals("To array should return all elements added.", elements, returned);
	}

}
